@@ -1,5 +1,5 @@
 # This file is part of Rubber and thus covered by the GPL
-# (c) Emmanuel Beffara, 2003--2004
+# (c) Emmanuel Beffara, 2003--2005
 """
 This is the command line pipe interface for Rubber.
 """
@@ -39,16 +39,14 @@ def dump_file (f_in, f_out):
 
 class Main (rubber.cmdline.Main):
 	def __init__ (self):
-		"""
-		Create the object used for message output.
-		"""
-		self.msg = rubber.cmdline.Message(level=-1)
+		rubber.cmdline.Main.__init__(self)
+		msg.level = 0
 
 	def help (self):
 		"""
 		Display the description of all the options and exit.
 		"""
-		self.msg (-1, _("""\
+		msg (0, _("""\
 This is Rubber version %s.
 usage: rubber-pipe [options]
 available options:
@@ -56,9 +54,11 @@ available options:
   -e, --epilogue=CMD       run the directive CMD after parsing
   -z, --gzip               compress the final document
   -h, --help               display this help
+      --into=DIR           go to directory DIR before compiling
   -k, --keep               keep the temporary files after compiling
   -l, --landscape          change paper orientation (if relevant)
   -m, --module=MOD[:OPTS]  use module MOD (with options OPTS)
+      --only=SOURCES       only include the specified SOURCES
   -o, --post=MOD[:OPTS]    postprocess with module MOD (with options OPTS)
   -d, --pdf                compile with pdftex (synonym for -m pdftex)
   -p, --ps                 process through dvips (synonym for -m dvips)
@@ -73,7 +73,7 @@ available options:
 	def parse_opts (self, cmdline):
 		args = rubber.cmdline.Main.parse_opts(self, cmdline)
 		if len(args) > 0:
-			self.msg(0, _("warning: the following options were ignored: %s")
+			msg.warn(_("the following options were ignored: %s")
 				% string.join(args, " "))
 				
 	def main (self, cmdline):
@@ -81,58 +81,72 @@ available options:
 		Run Rubber as a pipe for the specified command line. This dumps the
 		standard input into a temporary file, compiles it, dumps the result on
 		standard output, and then removes the files if requested. If an error
-		happens while building the document, the process stops.  The method
+		happens while building the document, the process stops. The method
 		returns the program's exit code.
 		"""
 		self.prologue = []
 		self.epilogue = []
 		self.clean = 1
+		self.place = "."
+		self.path = []
 		self.parse_opts(cmdline)
-		self.msg(2, _("This is Rubber version %s.") % version)
+		msg.log(_("This is Rubber version %s.") % version)
 
 		# Put the standard input in a file
 
+		if self.place is not None and self.place != ".":
+			dir = os.getcwd()
+			self.path = map(os.path.abspath, self.path)
+			self.path.insert(0, dir)
+			os.chdir(self.place)
+
 		src = make_name() + ".tex"
 		try:
 			srcfile = open(src, 'w')
 		except IOError:
-			self.msg(-1, _("cannot create temporary files"))
+			msg.error(_("cannot create temporary files"))
 			return 1
 
-		self.msg(1, _("saving the input in %s...") % src)
+		msg.progress(_("saving the input in %s") % src)
 		dump_file(sys.stdin, srcfile)
 		srcfile.close()
 
 		# Make the document
 
-		env = Environment(self.msg)
+		env = Environment()
 
 		if env.set_source(src):
-			sys.exit(1)
+			msg.error(_("cannot open the temporary %s") % src)
+			return 1
 
-		for cmd in self.prologue:
-			cmd = string.split(cmd, maxsplit = 1)
-			if len(cmd) == 1:
-				cmd.append("")
-			env.command(cmd[0], cmd[1], {'file': 'command line'})
+		if self.include_only is not None:
+			env.main.includeonly(self.include_only)
 
 		env.make_source()
-		env.parse()
+
+		for dir in self.path:
+			env.main.do_path(dir)
+		for cmd in self.prologue:
+			cmd = parse_line(cmd, {})
+			env.main.command(cmd[0], cmd[1:], {'file': 'command line'})
+
+		env.main.parse()
 
 		for cmd in self.epilogue:
-			cmd = string.split(cmd, maxsplit = 1)
-			if len(cmd) == 1:
-				cmd.append("")
-			env.command(cmd[0], cmd[1], {'file': 'command line'})
+			cmd = parse_line(cmd, {})
+			env.main.command(cmd[0], cmd[1:], {'file': 'command line'})
 
 		ret = env.final.make()
 
 		if ret == 0:
-			if not self.msg.short:
-				self.msg(1, _("There were errors."))
-			env.final.failed().show_errors()
+			msg.info(_("There were errors."))
+			try:
+				env.final.failed().show_errors()
+			except rubber.cmdline.MoreErrors:
+				msg.info(_("More errors."))
 			return 1
 
+
 		# Dump the results on standard output
 
 		output = open(env.final.prods[0])
@@ -141,7 +155,7 @@ available options:
 		# Clean the intermediate files
 
 		if self.clean:
-			env.clean(1)
+			env.final.clean()
 			os.unlink(src)
 		return 0
 
@@ -149,5 +163,5 @@ available options:
 		try:
 			return self.main(cmdline)
 		except KeyboardInterrupt:
-			self.msg(0, _("*** interrupted"))
+			msg.error(_("*** interrupted"))
 			return 2
