@@ -1,5 +1,5 @@
 # This file is part of Rubber and thus covered by the GPL
-# (c) Emmanuel Beffara, 2002--2004
+# (c) Emmanuel Beffara, 2002--2005
 """
 This is the command line interface for Rubber.
 """
@@ -9,90 +9,36 @@ import os.path
 import string
 from getopt import *
 
-from rubber import _
+from rubber import _, msg
 from rubber import *
 from rubber.version import *
+from rubber.util import parse_line
 
 class MoreErrors:
 	"""
 	This exception is raised when the maximum number of displayed errors is
 	reached.
 	"""
-	def __init__ (self, where):
-		self.where = where
-
-class Message (Message):
-	"""
-	This class defines a message writer that outputs the messages on standard
-	error according to GNU conventions. It manages verbosity level and
-	possible redirection.
-	"""
-	def __init__ (self, level=0, short=0):
-		"""
-		Initialize the object with the specified verbosity threshold. The
-		other argument is a boolean that indicates whether error messages
-		should be short or normal.
-		"""
-		self.level = level
-		self.short = short
-		self.write = self.do_write
-		self.max_errors = 10
-
-	def do_write (self, level, text):
-		if level <= self.level:
-			sys.stderr.write(text + "\n")
-
-	def __call__ (self, level, text):
-		self.write(level, text)
-
-	def format_pos (self, where):
-		if where.has_key("file"):
-			text = simplify_path(where["file"])
-			if where.has_key("line") and where["line"]:
-				text = "%s:%d" % (text, where["line"])
-				if where.has_key("last"):
-					if where["last"] != where["line"]:
-						text = "%s-%d" % (text, where["last"])
-		else:
-			text = "%r" % where
-			#_("(nowhere)")
-		return text
-
-	def error (self, where, text, code=None):
-		self.max_errors = self.max_errors - 1
-		if self.max_errors == -1:
-			raise MoreErrors(where)
-		prefix = self.format_pos(where) + ": "
-		if text[0:13] == "LaTeX Error: ":
-			text = text[13:]
-		self.write(-1, prefix + text)
-		if code and not self.short:
-			self.write(-1, prefix + _("leading text: ") + code)
-
-	def abort (self, what, why):
-		if self.short:
-			self.write(0, _("compilation aborted ") + why)
-		else:
-			self.write(0, _("compilation aborted: %s %s") % (what, why))
-
-	def info (self, where, what):
-		text = self.format_pos(where) + ": " + what
-		if where.has_key("page"):
-			text = "%s (page %d)" % (text, where["page"])
-		self.write(0, text)
+	pass
 
 class Main (object):
 	def __init__ (self):
-		"""
-		Create the object used for message output.
-		"""
-		self.msg = Message()
+		self.max_errors = 10
+		self.include_only = None
+		msg.write = self.stderr_write
+
+	def stderr_write (self, text, level=0):
+		if level <= 0:
+			self.max_errors = self.max_errors - 1
+			if self.max_errors == -1:
+				raise MoreErrors()
+		sys.stderr.write(text + "\n")
 
 	def short_help (self):
 		"""
 		Display a short description of the command line.
 		"""
-		self.msg (0, _("""\
+		msg(0, _("""\
 usage: rubber [options] sources...
 For more information, try `rubber --help'."""))
 
@@ -100,7 +46,7 @@ For more information, try `rubber --help'."""))
 		"""
 		Display the description of all the options and exit.
 		"""
-		self.msg (0, _("""\
+		msg(0, _("""\
 This is Rubber version %s.
 usage: rubber [options] sources...
 available options:
@@ -110,33 +56,38 @@ available options:
   -f, --force              force at least one compilation
   -z, --gzip               compress the final document
   -h, --help               display this help
+      --inplace            compile the documents from their source directory
+      --into=DIR           go to directory DIR before compiling
   -l, --landscape          change paper orientation (if relevant)
   -n, --maxerr=NUM         display at most NUM errors (default: 10)
   -m, --module=MOD[:OPTS]  use module MOD (with options OPTS)
+      --only=SOURCES       only include the specified SOURCES
   -o, --post=MOD[:OPTS]    postprocess with module MOD (with options OPTS)
-  -d, --pdf                compile with pdftex (synonym for -m pdftex)
+  -d, --pdf                produce a pdf (synonym for -m pdftex or -o ps2pdf)
   -p, --ps                 process through dvips (synonym for -o dvips)
   -q, --quiet              suppress messages
   -r, --read=FILE          read additional directives from FILE
   -s, --short              display errors in a compact form
   -I, --texpath=DIR        add DIR to the search path for LaTeX
   -v, --verbose            increase verbosity
-      --version            print version information and exit\
+      --version            print version information and exit
+  -W, --warn=TYPE          report warnings of the given TYPE (see man page)\
 """) % version)
 
 	def parse_opts (self, cmdline, short="", long=[]):
 		try:
 			opts, args = getopt(
-				cmdline, "I:c:de:fhklm:n:o:pqr:svz" + short,
+				cmdline, "I:c:de:fhklm:n:o:pqr:svW:z" + short,
 				["clean", "command=", "epilogue=", "force", "gzip", "help",
-				 "keep", "landcape", "maxerr=", "module=", "post=", "pdf",
-				 "ps", "quiet", "read=", "short", "texpath=", "verbose",
-				 "version"] + long)
+				 "inplace", "into=", "keep", "landcape", "maxerr=", "module=",
+				 "only=", "post=", "pdf", "ps", "quiet", "read=", "short",
+				 "texpath=", "verbose", "version", "warn="] + long)
 		except GetoptError, e:
 			print e
 			sys.exit(1)
 
 		extra = []
+		using_dvips = 0
 
 		for (opt,arg) in opts:
 			if opt == "--clean":
@@ -152,36 +103,58 @@ available options:
 			elif opt in ("-h", "--help"):
 				self.help()
 				sys.exit(0)
+			elif opt == "--inplace":
+				self.place = None
+			elif opt == "--into":
+				self.place = arg
 			elif opt in ("-k", "--keep"):
 				self.clean = 0
 			elif opt in ("-l", "--landscape"):
 				self.prologue.append("paper landscape")
 			elif opt in ("-n", "--maxerr"):
-				self.msg.max_errors = int(arg)
+				self.max_errors = int(arg)
 			elif opt in ("-m", "--module"):
 				self.prologue.append("module " +
 					string.replace(arg, ":", " ", 1))
+			elif opt == "--only":
+				self.include_only = arg.split(",")
 			elif opt in ("-o", "--post"):
 				self.epilogue.append("module " +
 					string.replace(arg, ":", " ", 1))
 			elif opt in ("-d", "--pdf"):
-				self.prologue.append("module pdftex")
+				if using_dvips:
+					self.epilogue.append("module ps2pdf")
+				else:
+					self.prologue.append("module pdftex")
 			elif opt in ("-p", "--ps"):
 				self.epilogue.append("module dvips")
+				using_dvips = 1
 			elif opt in ("-q", "--quiet"):
-				self.msg.level = -1
+				msg.level = msg.level - 1
 			elif opt in ("-r" ,"--read"):
 				self.prologue.append("read " + arg)
 			elif opt in ("-s", "--short"):
-				self.msg.short = 1
+				msg.short = 1
 			elif opt in ("-I", "--texpath"):
-				self.prologue.append("path " + arg)
+				self.path.append(arg)
 			elif opt in ("-v", "--verbose"):
-				self.msg.level = self.msg.level + 1
+				msg.level = msg.level + 1
 			elif opt == "--version":
 				print "Rubber version: " + version
 				print "module path: " + moddir
 				sys.exit(0)
+			elif opt in ("-W", "--warn"):
+				self.warn = 1
+				if arg == "all":
+					self.warn_boxes = 1
+					self.warn_misc = 1
+					self.warn_refs = 1
+				if arg == "boxes":
+					self.warn_boxes = 1
+				elif arg == "misc":
+					self.warn_misc = 1
+				elif arg == "refs":
+					self.warn_refs = 1
 
 			elif arg == "":
 				extra.append(opt)
@@ -201,65 +174,109 @@ available options:
 		self.epilogue = []
 		self.clean = 0
 		self.force = 0
+
+		self.warn = 0
+		self.warn_boxes = 0
+		self.warn_misc = 0
+		self.warn_refs = 0
+
+		self.place = "."
+		self.path = []
+
 		args = self.parse_opts(cmdline)
-		self.msg(2, _("This is Rubber version %s.") % version)
+
+		if self.place != ".":
+			initial_dir = os.getcwd()
+			msg.cwd = join(initial_dir, "")
+			if self.place is not None:
+				msg.path = self.place
+				self.place = os.path.abspath(self.place)
+			self.path = map(os.path.abspath, self.path)
+
+		msg.log(_("This is Rubber version %s.") % version)
 
 		for src in args:
-			env = Environment(self.msg)
+			env = Environment()
+
+			# Go to the appropriate directory
+
+			if self.place != ".":
+				src = os.path.abspath(os.path.join(initial_dir, src))
+				if self.place is None:
+					msg.path = os.path.dirname(src)
+					os.chdir(os.path.dirname(src))
+					src = os.path.basename(src)
+				else:
+					os.chdir(self.place)
 
 			# Check the source and prepare it for processing
 	
 			if env.set_source(src):
-				sys.exit(1)
+				msg.error(_("cannot find %s") % src)
+				return 1
 
-			for cmd in self.prologue:
-				cmd = string.split(cmd, maxsplit = 1)
-				if len(cmd) == 1:
-					cmd.append("")
-				env.command(cmd[0], cmd[1], {'file': 'command line'})
+			if self.include_only is not None:
+				env.main.includeonly(self.include_only)
 
 			if self.clean:
-				if not os.path.exists(env.source()):
-					self.msg(1, _("there is no LaTeX source"))
+				if env.main.prods == []:
+					msg.warn(_("there is no LaTeX source for %s") % src)
 					continue
+			else:
+				env.make_source()
 
-			env.make_source()
-			env.parse()
+			for dir in self.path:
+				env.main.do_path(dir)
+			for cmd in self.prologue:
+				cmd = parse_line(cmd, {})
+				env.main.command(cmd[0], cmd[1:], {'file': 'command line'})
+
+			env.main.parse()
 
 			for cmd in self.epilogue:
-				cmd = string.split(cmd, maxsplit = 1)
-				if len(cmd) == 1:
-					cmd.append("")
-				env.command(cmd[0], cmd[1], {'file': 'command line'})
+				cmd = parse_line(cmd, {})
+				env.main.command(cmd[0], cmd[1:], {'file': 'command line'})
 
 			# Compile the document
 
 			if self.clean:
-				env.clean(1)
-			else:
-				if self.force:
-					ret = env.make(1)
-					if ret != 0:
-						ret = env.final.make()
-					else:
-						# This is a hack for the call to show_errors() below
-						# to work when compiling failed when using -f.
-						env.final.failed_dep = env
-				else:
-					ret = env.final.make(self.force)
+				env.final.clean()
+				continue
 
-				if ret == 1:
-					self.msg(0, _("nothing to be done for %s") % env.source())
-				elif ret == 0:
-					if not self.msg.short:
-						self.msg(1, _("There were errors compiling %s.")
-							% env.source())
-					try:
-						env.final.failed().show_errors()
-					except MoreErrors, e:
-						self.msg(0, _("More errors in %s.") %
-							simplify_path(e.where["file"]))
+			if self.force:
+				ret = env.main.make(1)
+				if ret != 0 and env.final is not env.main:
+					ret = env.final.make()
+				else:
+					# This is a hack for the call to show_errors() below
+					# to work when compiling failed when using -f.
+					env.final.failed_dep = env.main
+			else:
+				ret = env.final.make(self.force)
+
+			if ret == 0:
+				msg.info(_("There were errors compiling %s.") % src)
+				try:
+					env.final.failed().show_errors()
+				except MoreErrors:
+					msg.info(_("More errors."))
+				return 1
+
+			if ret == 1:
+				msg(1, _("nothing to be done for %s") % src)
+
+			if self.warn:
+				log = env.main.log
+				self.max_errors = -1
+				if log.read(env.main.src_base + ".log"):
+					msg.error(_("cannot read the log file"))
 					return 1
+				if self.warn_boxes:
+					log.show_boxes()
+				if self.warn_refs:
+					log.show_references()
+				if self.warn_misc:
+					log.show_warnings()
 
 		return 0
 
@@ -275,5 +292,5 @@ available options:
 		try:
 			return self.main(cmdline)
 		except KeyboardInterrupt:
-			self.msg(0, _("*** interrupted"))
+			msg.error(_("*** interrupted"))
 			return 2
